/**
 * @file  DFFileDisplay.h
 * @brief Schnittstelle fuer die Klasse CDFFileDisplay.
 */
//////////////////////////////////////////////////////////////////////
#ifndef _DFFILEDISPLAY_H_
#define _DFFILEDISPLAY_H_

#ifdef _WIN32
#pragma warning(disable : 4996)
#endif

// Verwaltungsklasse fuer Datei
#include "DFFileIntelHex.h"
// 16bit/32bit CRC
#include "../Common/crc.h"

// Strukturdefinitionen
#include "../Common/pzemaiv.h"

// Maximale Anzahl Datenbyte (128kByte)
#define MAX_MEM_FONT   0x10000
//#define MAX_MEM_GRAFIC 0x10000
#define MAX_MEM_GRAFIC 0x8000

// Maximale Anzahl Zeichensaetze
#define MAX_FONT   10
// Maximale Anzahl der Grafiken
#define MAX_GRAFIC 250

class CDFFileDisplay : public CDFFileIntelHex
{
public:
	CDFFileDisplay();
	virtual ~CDFFileDisplay();

public:
	unsigned int m_sizeFont;
	unsigned int m_sizeGrafic;
	unsigned int m_sizeData;

	// Verwaltungsstrukturen fuer Zeichensatz
	FONTNODE *m_pFontNode;
	FONT     *m_pFont[MAX_FONT];

	// Verwaltungsstrukturen fuer Grafik
	GRAFICNODE *m_pGraficNode;
	GRAFIC     *m_pGrafic[MAX_GRAFIC];

	// Speicher fuer alle Daten
	unsigned char m_memFont[MAX_MEM_FONT];
	unsigned char m_memGrafic[MAX_MEM_GRAFIC];

public:
	void Reset();
	int Load(char *fileName);
	int Save(char *fileName, unsigned char append = 0);
	inline unsigned int GetSizeData() { return m_sizeData; };
	void Default(int iFontCount, int iGraficCount);
	int AppendSegmentText(unsigned char *buf, unsigned int size);
	void FontCorrect(int iNum);


protected:
  void DefaultFont(unsigned char count);
  void DefaultGrafic(unsigned char count);

protected:
	int LoadFont(char *fileName);
	int LoadGrafic(char *fileName);
	int SaveFont(char *fileName, unsigned char append = 0);
	int SaveGrafic(char *fileName, unsigned char append = 0);
};

#endif

