{----------------------------------------------------------------------------------------------------------------------}
{ DATAFOX GMBH (2005) 36124 Eichenzell, Sachsenhausen 3-7, e-mail:  www.datafox.de                                     }
{----------------------------------------------------------------------------------------------------------------------}
{    Hilfsfunktionen fr Datafox Testtools                                                                             }
{                                                                                                                      }
{   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~                                                            }
{    Autor          :  Jens Leuthold  (JML)                                                                            }
{   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~                                                            }
{    Erstellt am    :  11.10.2004                                                                                      }
{    Erstellt mit   :  Borland Delphi 7 Architect                                                                      }
{   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~                                                            }
{    Unit           :  DFUtility.pas                                                                                   }
{    Version        :  1.00                                                                                            }
{ ---------------------------------------------------------------------------------------------------------------------}
{    Art der Auflistung                       : none                                                                   }
{    Verwendete Anzeigebreite                 : 120 Zeichen                                                            }
{                                                                                                                      }
{ ---------------------------------------------------------------------------------------------------------------------}
{                                                                                                                      }
{ Der Quellcode darf frei verwendet werden. Eine Haftung fr den Einsatz dieser Software und Teilen des Quellcodes     }
{ wird vom Autor ausgeschlossen.                                                                                       }
{----------------------------------------------------------------------------------------------------------------------}
unit DFUtility;
interface

uses

  SysUtils,
  StdCtrls,
  ComCtrls,
  Windows,
  Classes,
  Controls,
  Messages,
  Registry;


type TByMethod = (bmWinAPI, bmWinReg);
function SerialPortList(Sender: TObject; ByMethod: TByMethod; MaxPort: Integer =32 ):Integer;
procedure SetControlToPanel(Control: TWincontrol; StatusBar: TStatusBar; PanelId: Integer);

//----------------------------------------------------------------------------------------------------------------------
implementation
uses
  Math;
{----------------------------------------------------------------------------------------------------------------------
| Die Funktion ermittelt die vorhandenen Serial-Ports des Systems (only Windows)                                       |
|----------------------------------------------------------------------------------------------------------------------|
| PARAMETER     |  Sender:      Komponente an die das Ergebnis der Routine geht (erlaubt sind TListBox, TComboBox      |
|               |               und TStringList). Wird eine andere Komponente bergeben, gibt die Funktion nur         |
|               |               die Anzahl der ermittelten Schnittstellen als Integer zurck. Wird TListBox,           |
|               |               TComboBox oder TStringList bergeben, wird ihr bestehender Inhalt gelscht.            |
|               |                                                                                                      |
|               |                                                                                                      |
|               |  ByMethod:    Gibt die Methode an, wie die Serial-Ports ermittelt werden.                            |
|               |               -  bmWinAPI:   die Ports werden mit Hilfe der API-Funktionen ermittelt                 |
|               |               -  bmWinReg:   die Ports werden aus der Registry ermittelt                             |
|               |                                                                                                      |
|               |  MaxPort:     Legt die Anzahl der Ports fest die maximal zurckgeliefert werden (Standard 32).       |
|               |               Dieser Wert hat nur Auswirkung wenn die Methode "bmWinAPI" benutzt wird. Dieser        |
|               |               Wert hat grossen Einfluss auf die Ausfhrungsgeschwindigkeit, er sollte nicht          |
|               |               zu gross gewhlt werden. COM-Ports grer 32 sind wohl eher selten.                    |
|----------------------------------------------------------------------------------------------------------------------|
| RCKGABEWERT  |  Liefert die Anzahl der gefundenen Serial-Ports zurck                                               |
|               |                                                                                                      |
|               |    -1         Fehler bei Ausfhrung                                                                  |
|               |     0         kein Serial-Port gefunden                                                              |
|               |    >0         Anzahl der gefundenen Ports                                                            |
|----------------------------------------------------------------------------------------------------------------------|
| HINWEIS       |  Die Ergebnisse dieser Funktion sind nicht immer korrekt. Manche Ports werden wegen installierter    |
|               |  Treiber nicht erkannt, andere Ports (z.B. bei der PCMCIA Fritz-Card) werden bei der Entfernung      |
|               |  aus dem Computer nicht aus der Registry entfernt.                                                   |
|               |  Es werden alle ermittelten Ports zurckgeliefert, auch wenn diese schon in Benutzung sind und       |
|               |  fr eine Verwendung aktuell nicht zur Verfgung stehen.                                             |
 ----------------------------------------------------------------------------------------------------------------------}
function SerialPortList(Sender: TObject; ByMethod: TByMethod; MaxPort: Integer =32 ):Integer;
var
  Reg         : TRegistry;
  ComPortList : TStringList;
  i           : Integer;
  BufSize     : Cardinal;
  DeviceName  : array[0..7] of char;
  CommConfig  : TCommConfig;
  AvailPorts  : Boolean;
begin
  Result:=-1;
  ComPortList :=   TStringList.Create;

  case ByMethod of
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    bmWinAPI:
    begin
      try
        DeviceName:='COMx'+#0+#0;
        BufSize:=SizeOf(CommConfig);
        for i:=0 to MaxPort-1 do begin
          if i<9 then begin
            DeviceName[3]:=chr($31+i);                   // $31 -> 1
            end
            else begin
              DeviceName[3]:=chr($30+((i+1) div 10));    // $30 -> 0
              DeviceName[4]:=chr($30+((i+1) mod 10));    // $30 -> 0
          end;
          AvailPorts:=GetDefaultCommConfig(DeviceName,CommConfig,BufSize);
            if AvailPorts then begin
               ComPortList.Add('COM'+IntToStr(i+1));
            end;
        end;
        finally
        Result:=-1;
      end;
    end;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    bmWinReg:
    begin
      Reg := TRegistry.Create;
      try
        // Registry Eintrg fr serielle nach ComPortList bertragen
        Reg.RootKey := HKEY_LOCAL_MACHINE;
        Reg.OpenKey('HARDWARE\DEVICEMAP\SERIALCOMM', False);
        Reg.GetValueNames(ComPortList);
        for i:=0 to ComPortList.Count - 1 do begin
          ComPortList[i]:=Reg.ReadString(ComPortList[i]);
        end;
        finally
          Result:=-1;
          Reg.Free;
      end;
    end;
  end;

  ComPortList.Sorted:=True;

  // wenn Sender ListBox
  if(Sender is TListBox)and (ComPortList.Count>0) then begin
    (Sender as TListBox).Items.Clear;
    for i:=0 to ComPortList.Count - 1 do begin
      (Sender as TListBox).Items.Add(ComPortList[i]);
    end;
  end;

  // wenn Sender ComboBox
  if(Sender is TComboBox) and (ComPortList.Count>0) then begin
    (Sender as TComboBox).Items.Clear;
    for i:=0 to ComPortList.Count - 1 do begin
      (Sender as TComboBox).Items.Add(ComPortList[i]);
    end;
  end;

  // wenn Sender StringList
  if(Sender is TStringList) and (ComPortList.Count>0) then begin
    (Sender as TStringList).Clear;
    for i:=0 to ComPortList.Count - 1 do begin
      (Sender as TStringList).Add(ComPortList[i]);
    end;
  end;

  if (ComPortList.Count>=0) then Result:=ComPortList.Count;
  ComPortList.Free;
end;
//----------------------------------------------------------------------------------------------------------------------
procedure SetControlToPanel(Control: TWincontrol; StatusBar: TStatusBar; PanelId: Integer);
// welche Komponente  // auf welchen Statusbar  // in welches Panel
var ARect : TRect;
begin
  StatusBar.Perform((WM_USER + 10), PanelId, Integer(@ARect));
  Control.Parent := StatusBar;
  Control.Top    := ARect.Top;
  Control.Left   := ARect.Left;
  Control.Width  := ARect.Right-ARect.Left;
  Control.Height := ARect.Bottom-ARect.Top;
end;
//----------------------------------------------------------------------------------------------------------------------
end.
