/*
 * DFCom.java
 * 
 */
package at.datafox.dfcom.spi.type2;

import java.nio.charset.Charset;

public final class DFCom
{

  public static final Charset DEFAULT_CHARSET = Charset.forName("ISO-8859-1");
  public static final int BUSID_HOST = 254;
  public static final int COMTYPE_RS232 = 1;
  public static final int COMTYPE_WIRELESS = 2;
  public static final int COMTYPE_SOCKET = 3;
  public static final int AUDIO_OFF = 0;
  public static final int AUDIO_BEEP = 1;
  public static final int AUDIO_DOUBLE_BEEP = 2;

  public static native boolean dfcComOpenSerial(int channel, byte[] comPort, int baudRate,
                                                int timeout);

  public static native boolean dfcComOpenSocket(int channel, byte[] host, int port, int timeout);

  public static native boolean dfcComOpenIV(int channel, int busId, int type, byte[] comOrHost,
                                            int baudOrPort, int timeOut);

  public static native void dfcComClose(int channel);

  public static native long dfcCheckDevice(int channel, int busId, int retry);

  public static native boolean dfcComSetTime(int channel, int busId, byte[] timeParts);

  public static native byte[] dfcComGetTime(int channel, int busId);

  public static native boolean dfcComSendMessage(int channel, int busId, int displayTime, int action,
                                                 int audio, byte[] message);

  public static native boolean dfcComSendInfoText(int channel, int busId, byte[] message);

  public static class SerialNumberResult
  {

    public long errorCode;
    public long serialCode;

    SerialNumberResult(long pErrorLog, long pSerialCode)
    {
      errorCode = pErrorLog;
      serialCode = pSerialCode;
    }
  };

  public static native SerialNumberResult dfcGetSeriennummer(int channel, int busId);

  public static native void dfcSetLog(int channel, boolean onoff);

  public static native void dfcSetLogFilename(int channel, byte[] fileName);

  public static interface CallBack
  {

    public boolean onProgress();
  }

  public static native void dfcSetCallBack(int channel, CallBack callBack);

  public static native long dfcSetGlobVar(int channel, int busId, byte[] varName, int type,
                                          byte[] value);

  public static class GetGlobalVarResult
  {

    public long errorCode;
    public String value;

    GetGlobalVarResult(long pErrCode, byte[] pValue)
    {
      errorCode = pErrCode;
      value = new String(pValue, DEFAULT_CHARSET);
    }
  }

  public static native GetGlobalVarResult dfcGetGlobVar(int channel, int busId, byte[] varName,
                                                        int type);

  public static native long dfcCloseRelay(int channel, int busId, byte relay, int time);

  public static class GetRelayStateResult
  {

    public long errorCode;
    public boolean closed;
    public int time;

    GetRelayStateResult(long pErrCode, int pTime, boolean pClosed)
    {
      errorCode = pErrCode;
      time = pTime;
      closed = pClosed;
    }
  }

  public static native GetRelayStateResult dfcGetRelayState(int channel, int busId, byte relay);

  public static native long dfcOpenRelay(int channel, int busId, byte relay);

  public static native int dfcGetDevicePollRetry(int channel);

  public static class GetVersionFirmwareResult
  {

    public long errorCode;
    public String version;

    GetVersionFirmwareResult(long pErrorCode, byte[] pVersion)
    {
      version = new String(pVersion, DEFAULT_CHARSET);
      errorCode = pErrorCode;
    }
  }

  public static native GetVersionFirmwareResult dfcGetVersionFirmware(int channel, int bus);
  public static native GetVersionFirmwareResult dfcGetVersionFromFile(int channel,byte[] fileName);
  public static class WriteResult
  {

    public long errorCode;
    public int written;

    WriteResult(long pErrorCode, int pWritten)
    {
      written = pWritten;
      errorCode = pErrorCode;
    }
  }

  public static native WriteResult dfcWrite(int channel, int bus, byte[] data);

  public static class ReadResult
  {

    public long errorCode;
    public byte[] buffer;

    ReadResult(long pErrorCode, byte[] pBuffer)
    {
      errorCode = pErrorCode;
      buffer = pBuffer;
    }
  }

  public static native ReadResult dfcRead(int channel, int bus, int toRead);

  public static native long dfcUpload(int channel,int bus,byte[] fileName);
  private DFCom()
  {
  }
}
